<?php
/**
 * File containing the Person abstract class.
 * 
 * @category Class
 * @package PHPSupportTickets
 * @author Ian Warner, <iwarner@triangle-solutions.com> 
 * @author Nicolas Connault, <nicou@sweetpeadesigns.com.au> 
 * @copyright (C) 2001 Triangle Solutions Ltd
 * @version CVS: $Id: abstract.Person.php,v 1.5 2005/08/01 13:30:29 nicconn Exp $
 * @since File available since Release 1.1.1.1
 * \\||
 */
/**
 * includes
 */
require_once('interface.DatabaseMirror.php');
require_once('static/static.Validate.php');

/**
 * This abstract class serves as a top-level definition for all 
 * organisational entities: projects, companies, departments and offices.
 * All these classes should overload most of the methods defined here, though
 * this is not absolutely necessary.
 * 
 * @package PHPSupportTickets
 */
abstract class Person implements DatabaseMirror {
    /**
     * ID number matching the DB entry
     * 
     * @access private 
     * @var int 
     */
    private $ID;
    
    /**
    * Person's Nickname.
    * 
    * @access private 
    * @var String 
    */
    private $nickname;
    
    /**
     * Person's UNIX timestamp of creation in DB.
     * 
     * @access private 
     * @var String 
     */
    private $timestamp;

    /**
     * Constructor: takes an array whose keys are attributes of the 
     * Person and values are their value. Can come from either a 
     * submitted entry form, or a returned row from the DB.
     * 
     * @access public 
     * @param array $ Fields
     */
    public function __construct($fields) {
        if (is_array($fields)) extract($fields); 
        // Sometimes we want to create a blank Person that gets filled
        // from the database afterwards
        if (isset($timestamp)) {
            $this->setTimestamp($timestamp);
        } else {
            $this->setTimestamp(time());
        }
        
        $this->setNickname($nickname);
        
        if (isset($ID)) {
            $this->setID($ID);
        }
    } 

    /**
     * This static function must be implemented by all objects that implement
     * the DatabaseMirror interface. It enables us to validate the user input
     * fields BEFORE creating the object or entering it into the DB.
     * 
     * @param array $fields 
     * @return boolean True if all fields are valid, array of arrays with error
     *         message and name of field if error is encountered.
     */
    public static function validate($fields) {
        if (is_array($fields)) {
            
            extract($fields);
            $errors = array();
            if ($nickname == "" || is_null($nickname)) {
                $errors[] = array("field" => "nickname", 
                        "message" => VALIDATE_EMPTY_NICKNAME);
            } elseif (Validate::checkAlpha($nickname) !== true) {
                $errors[] = Validate::checkAlpha($nickname, "nickname");
            }
            
            if (count($errors) == 0) {
                // If no errors, return true
                return true;
            } else {           
                // If any error was found, an array of errors is returned
                return $errors;
            }
        }
    } 

    /* ---- GETTERS ---- */

    /**
    * Returns the Person's nickname.
    * 
    * @return string
    * @access public
    */
    public function getNickname() {
        return $this->nickname;
    }

    /**
     * Returns the Person's UNIX timestamp of creation in DB.
     * 
     * @return int 
     * @access public 
     */
    public function getTimestamp() {
        return $this->timestamp;
    } 

    /* ---- SETTERS ---- */
   
    /**
    * Sets the Person's nickname.
    * 
    * @param string
    * @access public
    */
    public function setNickname($string) {
         $this->nickname = $string;
    }

    /**
     * Sets the Person's UNIX timestamp of creation in DB.
     * 
     * @param int $ 
     * @access public 
     */
    public function setTimestamp($int) {
        $this->timestamp = $int;
    } 

    /**
     * Person::setID()
     * 
     * @param  $ID 
     * @access public 
     * @return boolean true if successful
     */
    public function setID($ID) {
        // Perform validation first
        $this->ID = $ID;
    } 

    
    
    /**
     * Returns a formatted string made up of different fields.
     * E.g. <title> <fname> <mname> <lname>, <job_title> (job_title belongs
     * to a subclass, this why this method is only abstract).
     * THIS METHOD MUST BE IMPLEMENTED BY ALL CONCRETE SUB-CLASSES !
     * 
     * @abstract 
     */
    abstract function getFullDescription();
} 

?>