<?php
/**
 * File containing the Answer class.
 * 
 * @category Class
 * @package PHPSupportTickets
 * @author Ian Warner, <iwarner@triangle-solutions.com> 
 * @author Nicolas Connault, <nicou@sweetpeadesigns.com.au> 
 * @copyright (C) 2001 Triangle Solutions Ltd
 * @version CVS: $Id: class.Answer.php,v 1.5 2005/08/16 07:26:41 nicconn Exp $
 * @since File available since Release 1.1.1.1
 * \\||
 */
 
/**
* user defined includes 
* @include
*/
require_once('class.Note.php');
require_once('SQL/class.AnswerSQLBuilder.php');
require_once('static/static.Logger.php');
require_once('class.Answer.php');
/**
* user defined constants 
* @ignore
*/

/**
* Answer class.
* 
* @access public 
* @package PHPSupportTickets
*/
class Answer {
    // --- ATTRIBUTES ---
    
    /**
    * The database table to mirror this object
    * 
    * @access public
    * @static 
    * @var string 
    */
    public static $table = "answers";
    
    /**
    * This answer's ID in the DB
    * 
    * @access private 
    * @var int $ID 
    */
     private $ID = null;
    
    /**
    * The ID of the user who created this answer.
    * 
    * @access private 
    * @var int $user_ID 
    */
     private $user_ID = null;
    
    /**
    * The body of the Answer.
    * 
    * @access private 
    * @var string $body
    */
     private $body;
    
    /**
    * Answer's subject.
    * 
    * @access private 
    * @var String $subject
    */
     private $subject = null;        
    
     /**
     * Timestamp of creation of this answer.
     * 
     * @access private
     * @var int $timestamp
     */
     private $timestamp;
     
     /**
     * Ticket_ID of Answer.
     * 
     * @access private
     * @var int $ticket_ID
     */
     private $ticket_ID;
     
     /**
     * User's Rating of Answer.
     * 
     * @access private
     * @var int $rating
     */
     private $rating;
     
     /**
     * Companion object used for building SQL queries. Separates
     * class responsibilities from SQL logic.
     * 
     * @access public
     * @var object $SQLBuilder
     */
     public $SQLBuilder;
     
    // --- METHODS ---
    
    /**
    * Constructor: first call on parent class constructor.
    * 
    * @access public 
    * @param array $fields From a Posted form or a database row 
    * @return void 
    */
    public function __construct($fields) {
        $this->SQLBuilder = new AnswerSQLBuilder(); 
        if (is_array($fields)) {
            extract($fields);
            $this->setUser_ID($user_ID);
            // $this->setSubject($subject);
            $this->setTimestamp($timestamp);
            $this->setBody($body);
            $this->setRating($rating);
            $this->setTicket_ID($ticket_ID);
            
            if (isset($ID)) {
                $this->setID($ID);
            }
        } else { 
            Logger::logEvent(3, "Answer Operation", null, 
                    "Wrong argument type given to Answer::__construct", __FILE__, __LINE__);
        }
    } 
    
    /* --- DATABASEMIRROR METHODS --- */
        
    /**
    * This static function must be implemented by all objects that implement
    * the DatabaseMirror interface. It enables us to validate the user input
    * fields BEFORE creating the object or entering it into the DB.
    * 
    * @param array $fields
    * @return boolean True if all fields are valid, array of arrays with error
    *       message and name of field if error is encountered.
    */ 
    public static function validate($fields) {        
        if (is_array($fields)) {
            extract($fields);
            $errors = array();
            
            if (!isset($body) || trim($body) == '') {
                $errors[] = (array('message' => VALIDATE_EMPTY_BODY, 'field' => 'body'));
            }
            // Verify that required fields are not empty
            // @todo Implement validation
            
            if (count($errors) == 0) {
                Logger::logEvent(3, "Answer Operation", null, 
                        "Answer object created, passed validation", __FILE__, __LINE__);
                return true;
            } else {           
                Logger::logEvent(3, "Answer Operation", null, 
                        "Answer object not created, failed validation", __FILE__, __LINE__);
                return $errors;
            }
        }
    }
    
    /**
    * Retrieves this Answer's ID number based on a unique variable.
    * Also sets this object's ID number to the retrieved int
    * 
    * @return int Answer's ID number
    */
    public function getID() {
        if (!isset($this->ID)) {
            $conn = &ADONewConnection(DSN);
            $query = $this->SQLBuilder->search(Answer::$table, array("ID"), 
                array("answers.timestamp" => $this->getTimestamp()));
            
            $rs = &$conn->Execute($query);
            
            if ($rs != false && $rs->RecordCount() > 0) {
                $this->setID($rs->fields['ID']);
                return $rs->fields['ID'];            
            } else {
                Logger::logEvent(3, "DB Error", null, 
                        "Could not retrieve Answer ID: " . $this->getID(), __FILE__, __LINE__);
            }
        } else {
            return $this->ID;
        } 
    }
    
    /**
    * Short description of method add
    * 
    * @access public 
    * @return boolean 
    */
    public function addToDB() {
        // Initialise Database object
        $conn = &ADONewConnection(DSN);
        
        // First check that Answer doesn't already exist.
        $query = $this->SQLBuilder->search(Answer::$table, array("ID"), 
                array("answers.timestamp" => $this->getTimestamp(), "ticket_ID" => $this->getTicket_ID()));
        $rs = &$conn->Execute($query);
        
        if ($rs->RecordCount() == 0) {
            // Retrieve array of table's column names (always UPPERCASE)
            $columns = $conn->MetaColumnNames(Answer::$table);
                        
            $query = $this->SQLBuilder->insert($this, $columns);
            $rs = &$conn->Execute($query);
            
            // Verify previous query
            if ($rs != false) {
                Logger::logEvent(3, "Answer Operation", null, 
                        "Answer object added to DB", __FILE__, __LINE__);
                return true;
            } else {                
                Logger::logEvent(3, "DB Error", null, 
                        "Could not Insert Answer object.", __FILE__, __LINE__);
                return false;
            }
        } else {
            Logger::logEvent(3, "Answer Operation", null, 
                        "Answer object not added to DB, username already exists", __FILE__, __LINE__);
            return false; 
        }
    }
    
    /**
    * Removes a Answer from the Database
    * 
    * @access public 
    * @param int $ID
    * @return boolean true if successful
    */
    public function removeFromDB($ID) {
        $conn = &ADONewConnection(DSN);
        $query = $this->SQLBuilder->delete($ID);
        $rs = &$conn->Execute($query);
        
        if ($conn->Affected_Rows() > 0) {
            Logger::logEvent(3, "Answer Operation", null, 
                    "Answer object deleted from DB.", __FILE__, __LINE__);
            return true;
        } else {
            Logger::logEvent(3, "DB Error", null, 
                    "Answer object could not be deleted", __FILE__, __LINE__);
            return false;
        }
    }
    
    /**
     * Implementation of the DatabaseMirror interface function getFromDB().
     * Returns a single Answer object from the DB based on input in $fields array
     * 
     * @static
     * @access public
     * @return Object Answer if successful, false otherwise
     */    
    public static function getFromDB($fields) {              
        if (is_array($fields)) {
            $myAnswer = null;
            
            $conn = &ADONewConnection(DSN);
            $conn->SetFetchMode(ADODB_FETCH_ASSOC);
            $SQLBuilder = new AnswerSQLBuilder();
            $query = $SQLBuilder->get($fields);

            if (!$query) {
                Logger::logEvent(3, "Answer Operation", null, 
                    "Wrong arguments passed to Answer::getFromDB()", __FILE__, __LINE__);
            }
            
            $rs = &$conn->Execute($query);
            
            if ($rs != false && $rs->RecordCount() > 0) {
                Logger::logEvent(3, "Answer Operation", null, 
                        "Answer object retrieved from DB", __FILE__, __LINE__);
                $myAnswer = new Answer($rs->fields);
            } else {
                Logger::logEvent(3, "DB Error", null, 
                        "Answer object could not be retrieved from DB", __FILE__, __LINE__);
                return false;
            }
        } else {
            Logger::logEvent(3, "Answer Operation", null, 
                    "Wrong argument type passed to Answer::getFromDB()", __FILE__, __LINE__);
            return false;
        }
        return $myAnswer;        
    }
    
    /**
    * Returns an array of Answer objects
    * 
    * @static
    * @access public
    * 
    * @param string $user_type Admin or Mod
    * @param string $search The search term
    * @param string $field The field being searched
    * @param string $sort The field by which to sort the results
    * @param string $order The order in which to sort the results
    * 
    * @return array of Answers
    */
    public static function getArrayFromDB($user_type = "Mod", $search = "", $field = ""
            , $sort = "", $order = "") {
        $conn = &ADONewConnection(DSN);
        $answers = array();
        $IDs = array();
        
        $SQLBuilder = new AnswerSQLBuilder();
        
        // Get query from SQLBuilder object
        $query = $SQLBuilder->getFullAnswersArray($this->getID(), $user_type, 
                $search, $field, $sort, $order);
        // Perform the query
        $rs = &$conn->Execute($query);
        
        // Build answers array from query result (use pagination)
        if ($rs != false && $rs->RecordCount() > 0) {
            $answer_count = 0;
            while (!$rs->EOF) {
                $row = $rs->fields; 
                // Increment answer count if this answer is new
                if (!isset($IDs[$row['ID']])) {
                    $answer_count++;
                } 
            
                // prepare arrays
                $answer_fields = array();
                $user_fields = array();
                $ticket_fields = array();
                $department_fields = array(); 
                
                // Increment 'recent' index count if answer within 1 week
                if ($row['timestamp'] >= (time() - 604860)) {
                    $answers[$row['ID']]['recent'] = true;
                } else {
                    $answers[$row['ID']]['recent'] = false;
                } 

                foreach ($row as $field => $value) {
                    if (eregi('^user_', $field)) {
                        $user_fields[substr($field, 5)] = $value;
                    } elseif (eregi('^ticket_', $field) && $value != null) {
                        $ticket_fields[substr($field, 7)] = $value;
                    } elseif (eregi('^department_', $field)) {
                        $department_fields[substr($field, 11)] = $value;
                    } else {
                        $answer_fields[$field] = $value;
                    } 
                } 

                $answers[$row['ID']]['ticket'] = $ticket_fields;
                $answers[$row['ID']]['answer'] = $answer_fields;
                $answers[$row['ID']]['user'] = $user_fields;
                $answers[$row['ID']]['department'] = $department_fields;
                $IDs[$row['ID']] = 1;
                $answers['count'] = $answer_count;
                $rs->MoveNext();
            } 
            return $answers;
        } else {
            Logger::logEvent(3, "DB Error", null, 
                    "SQL error trying to get Array of users: " . addslashes($query), __FILE__, __LINE__);
            return false;
        }
    }
    
    /* ---- GETTERS ---- */
    
    /**
    * Returns the Answer's user ID.
    * 
    * @return int user_ID
    * @access public
    */
    public function getUser_ID() {
        return $this->user_ID;
    }
    
    /**
    * Returns the Answer's subject.
    * 
    * @return string Subject
    * @access public
    */
    public function getSubject() {
        return $this->subject;
    }
        
    /**
    * Returns the Answer's creation timestamp.
    * 
    * @return int Timestamp of creation
    * @access public
    */
    public function getTimestamp() {
        return $this->timestamp;
    }
        
    /**
    * Returns the Answer's ticket_ID.
    * 
    * @return int ticket_ID
    * @access public
    */
    public function getTicket_ID() {
        return $this->ticket_ID;
    }
    
    /**
    * Returns the Answer's rating (open or closed).
    * 
    * @return string Rating
    * @access public
    */
    public function getRating() {
        return $this->rating;
    }
    
    /**
    * Returns the Answer's body.
    * 
    * @return string Body
    * @access public
    */
    public function getBody() {
        return $this->body;
    }
    
    /* ---- SETTERS ---- */
    
    /**
    * Sets the Answer's ID.
    * 
    * @param int $ID
    * @access public
    */
    public function setID($ID) {
        $this->ID = $ID;
    }
    
    /**
    * Sets the Answer's user ID.
    * 
    * @param int $user_ID
    * @access public
    */
    public function setUser_ID($user_ID) {
        $this->user_ID = $user_ID;
    }
    
    /**
    * Sets the Answer's subject.
    * 
    * @param string $subject
    * @access public
    */
    public function setSubject($subject) {
        $this->subject = $subject;
    }
        
    /**
    * Sets the Answer's creation timestamp.
    * 
    * @param int $timestamp
    * @access public
    */
    public function setTimestamp($timestamp) {
        $this->timestamp = $timestamp;
    }
        
    /**
    * Sets the Answer's ticket_ID.
    * 
    * @param int $ticket_ID
    * @access public
    */
    public function setTicket_ID($ticket_ID) {
        $this->ticket_ID = $ticket_ID;
    }
    
    /**
    * Sets the Answer's rating (open or closed).
    * 
    * @param string $rating
    * @access public
    */
    public function setRating($rating) {
        $this->rating = $rating;
    }
    
    /**
    * Sets the Answer's body.
    * 
    * @param string $body
    * @access public
    */
    public function setBody($body) {
        $this->body = $body;
    }
    
    /**
    * Tostring function: returns a string representation of this object
    * 
    * @access public
    * @return string
    */
    public function __toString() {
        $string = "Answer object:
            <br />___________________";
        $string .= "<br />ID: " . $this->getID();
        $string .= "<br />Subject: " . $this->getSubject();
        $string .= "<br />Body: " . $this->getBody();
        $string .= "<br />User_ID: " . $this->getUser_ID();
        $string .= "<br />Ticket_ID: " . $this->getTicket_ID();
        $string .= "<br />Date Created: " . gmdate('d-m-Y', $this->getTimestamp());
        $string .= "<br />Rating: " . $this->getRating();
        return $string;
    }
} 

?>
