<?php
/**
 * File containing the Ticket class.
 * 
 * @category Class
 * @package PHPSupportTickets
 * @author Ian Warner, <iwarner@triangle-solutions.com> 
 * @author Nicolas Connault, <nicou@sweetpeadesigns.com.au> 
 * @copyright (C) 2001 Triangle Solutions Ltd
 * @version CVS: $Id: class.Ticket.php,v 1.13 2005/08/17 08:47:33 nicconn Exp $
 * @since File available since Release 1.1.1.1
 * \\||
 */

/**
 * user defined includes
 * 
 * @include 
 */
require_once('class.Note.php');
require_once('SQL/class.TicketSQLBuilder.php');
require_once('static/static.Logger.php');
require_once('class.Answer.php');
/**
 * user defined constants
 * 
 * @ignore 
 */

/**
 * Ticket class.
 * 
 * @access public 
 * @package PHPSupportTickets
 */
class Ticket {
    // --- ATTRIBUTES ---
    /**
     * The database table to mirror this object
     * 
     * @access public 
     * @static 
     * @var string 
     */
    public static $table = "tickets";

    /**
     * This ticket's ID in the DB
     * 
     * @access private 
     * @var int $ID
     */
    private $ID = null;

    /**
     * The ID of the user who created this ticket.
     * 
     * @access private 
     * @var int $user_ID
     */
    private $user_ID = null;

    /**
     * The body of the Ticket.
     * 
     * @access private 
     * @var string $body
     */
    private $body;

    /**
     * Ticket's subject.
     * 
     * @access private 
     * @var String $subject
     */
    private $subject = null;

    /**
     * Ticket's status, open or closed
     * 
     * @access private 
     * @var String $status
     */
    private $status = 'open';

    /**
     * Urgency level of the Ticket, from 1 to 4
     * 
     * @access public 
     * @var int 
     */
    private $urgency = 1;

    /**
     * Ticket's category
     * 
     * @access private 
     * @var string $category
     */
    private $category = null;

    /**
     * Timestamp of creation of this ticket.
     * 
     * @access private 
     * @var int $timestamp
     */
    private $timestamp;

    /**
     * Department_ID of Ticket.
     * 
     * @access private 
     * @var int $department_ID
     */
    private $department_ID;

    /**
     * Ticket's array of Answer objects;
     * 
     * @access private 
     * @var array $answers
     */
    private $answers = array();

    /**
     * Companion object used for building SQL queries. Separates
     * class responsibilities from SQL logic.
     * 
     * @access public 
     * @var object $SQLBuilder
     */
    public $SQLBuilder; 
    // --- METHODS ---
    /**
     * Constructor: first call on parent class constructor.
     * 
     * @access public 
     * @param array $fields From a Posted form or a database row
     * @return void 
     */
    public function __construct($fields) {
        $this->SQLBuilder = new TicketSQLBuilder();
        if (is_array($fields) && count($fields) > 0) {
            extract($fields);
            $this->setUser_ID($user_ID);
            $this->setSubject($subject);
            $this->setTimestamp($timestamp);
            $this->setBody($body);
            $this->setStatus($status);
            $this->setUrgency($urgency);
            $this->setDepartment_ID($department_ID); 
            // if (!isset($body)) die(debug_print_backtrace());
            if (isset($ID)) {
                $this->setID($ID);
            } 
        } else {
            Logger::logEvent(3, "Ticket Operation", null,
                "Wrong argument type given to Ticket::__construct", __FILE__, __LINE__);
        } 
    } 

    /* --- DATABASEMIRROR METHODS --- */

    /**
     * This static function must be implemented by all objects that implement
     * the DatabaseMirror interface. It enables us to validate the user input
     * fields BEFORE creating the object or entering it into the DB.
     * 
     * @param array $fields 
     * @return boolean True if all fields are valid, array of arrays with error
     *        message and name of field if error is encountered.
     */
    public static function validate($fields) {
        if (is_array($fields)) {
            extract($fields);
            $errors = array(); 
            // Verify that required fields are not empty
            if ($subject == "" || is_null($subject)) {
                $errors[] = array("field" => "subject",
                    "message" => VALIDATE_EMPTY_SUBJECT);
            } 

            if ($body == "" || is_null($body)) {
                $errors[] = array("field" => "body",
                    "message" => VALIDATE_EMPTY_BODY);
            } 

            if (count($errors) == 0) {
                Logger::logEvent(3, "Ticket Operation", null,
                    "Ticket object created, passed validation", __FILE__, __LINE__);
                return true;
            } else {
                Logger::logEvent(3, "Ticket Operation", null,
                    "Ticket object not created, failed validation", __FILE__, __LINE__);
                return $errors;
            } 
        } 
    } 

    /**
     * Retrieves this Ticket's ID number based on a unique variable.
     * Also sets this object's ID number to the retrieved int
     * 
     * @return int Ticket's ID number
     */
    public function getID() {
        if (!isset($this->ID)) {
            $conn = &ADONewConnection(DSN);
            $query = $this->SQLBuilder->search(Ticket::$table, array("ID"),
                array("tickets.timestamp" => $this->getTimestamp()));

            $rs = &$conn->Execute($query);

            if ($rs != false && $rs->RecordCount() > 0) {
                $this->setID($rs->fields['ID']);
                return $rs->fields['ID'];
            } else {
                Logger::logEvent(3, "DB Error", null,
                    "Could not retrieve Ticket ID", __FILE__, __LINE__);
            } 
        } else {
            return $this->ID;
        } 
    } 

    /**
     * Short description of method add
     * 
     * @access public 
     * @return boolean 
     */
    public function addToDB() {
        // Initialise Database object
        $conn = &ADONewConnection(DSN); 
        // First check that Ticket doesn't already exist.
        $query = $this->SQLBuilder->search(Ticket::$table, array("ID"),
            array("tickets.timestamp" => $this->getTimestamp()));
        $rs = &$conn->Execute($query);

        if ($rs->RecordCount() == 0) {
            // Retrieve array of table's column names (always UPPERCASE)
            $columns = $conn->MetaColumnNames(Ticket::$table);

            $query = $this->SQLBuilder->insert($this, $columns);
            $rs = &$conn->Execute($query); 
            // Verify previous query
            if ($rs != false) {
                Logger::logEvent(3, "Ticket Operation", null,
                    "Ticket object added to DB", __FILE__, __LINE__);
                return true;
            } else {
                Logger::logEvent(3, "DB Error", null,
                    "Could not Insert Ticket object.", __FILE__, __LINE__);
                return false;
            } 
        } else {
            Logger::logEvent(3, "Ticket Operation", null,
                "Ticket object not added to DB, username already exists", __FILE__, __LINE__);
            return false;
        } 
    } 

    /**
     * Removes a Ticket from the Database
     * 
     * @access public 
     * @param int $ID 
     * @return boolean true if successful
     */
    public function removeFromDB($ID) {
        $conn = &ADONewConnection(DSN);
        $query = $this->SQLBuilder->delete($ID);
        $rs = &$conn->Execute($query);

        if ($conn->Affected_Rows() > 0) {
            Logger::logEvent(3, "Ticket Operation", null,
                "Ticket object deleted from DB.", __FILE__, __LINE__);
            return true;
        } else {
            Logger::logEvent(3, "DB Error", null,
                "Ticket object could not be deleted", __FILE__, __LINE__);
            return false;
        } 
    } 

    /**
     * Updates a Ticket in the Database
     * 
     * @access public 
     * @param int $ 
     * @return boolean true if successful
     */
    public function updateDB() {
        $conn = &ADONewConnection(DSN);
        $this->getID();

        $columns = $conn->MetaColumnNames(Ticket::$table);
        $query = $this->SQLBuilder->update($this, $columns);

        $rs = &$conn->Execute($query);

        if ($rs != false && $conn->Affected_Rows() > 0) {
            Logger::logEvent(3, "Ticket Operation", USER_USER_ID,
                "Ticket object updated in DB", __FILE__, __LINE__);
        } elseif ($conn->Affected_Rows() == 0) {
            Logger::logEvent(3, "Ticket Operation", USER_USER_ID,
                "Ticket object note updated, no changes to perform", __FILE__, __LINE__);
        } else {
            Logger::logEvent(3, "DB Error", USER_USER_ID,
                "Ticket object not updated, DB error", __FILE__, __LINE__);
            return false;
        } 

        return true;
    } 

    /**
     * Implementation of the DatabaseMirror interface function getFromDB().
     * Returns a single Ticket object from the DB based on input in $fields array
     * 
     * @static 
     * @access public 
     * @return Object Ticket if successful, false otherwise
     */
    public static function getFromDB($fields) {
        if (is_array($fields)) {
            $myTicket = null;

            $conn = &ADONewConnection(DSN);
            $conn->SetFetchMode(ADODB_FETCH_ASSOC);
            $SQLBuilder = new TicketSQLBuilder();
            $query = $SQLBuilder->get($fields);

            if (!$query) {
                Logger::logEvent(3, "Ticket Operation", null,
                    "Wrong arguments passed to Ticket::getFromDB()", __FILE__, __LINE__);
            } 

            $rs = &$conn->Execute($query);

            if ($rs != false && $rs->RecordCount() > 0) {
                Logger::logEvent(3, "Ticket Operation", null,
                    "Ticket object retrieved from DB", __FILE__, __LINE__);
                $myTicket = new Ticket($rs->fields);
            } else {
                Logger::logEvent(3, "DB Error", null,
                    "Ticket object could not be retrieved from DB", __FILE__, __LINE__);
                return false;
            } 
        } else {
            Logger::logEvent(3, "Ticket Operation", null,
                "Wrong argument type passed to Ticket::getFromDB()", __FILE__, __LINE__);
            return false;
        } 
        return $myTicket;
    } 

    /**
     * Returns an array of Ticket objects
     * 
     * @access public 
     * @param string $user_type Admin or Mod
     * @param string $search The search term
     * @param string $field The field being searched
     * @param string $sort The field by which to sort the results
     * @param string $order The order in which to sort the results
     * @param string $ticket_type 'department' tickets or 'my' tickets
     * @return array of Tickets
     */
    public static function getArrayFromDB($user_type, $search, $field,
        $sort, $order, $max_records, $rs_page, $department_ID, $ticket_type) {
        $conn = &ADONewConnection(DSN);
        $tickets = array();
        $IDs = array();

        $SQLBuilder = new TicketSQLBuilder();
        if (!isset($rs_page)) {
            $rs_page = 1;
        } 
        $tickets['count'] = 0; 
        
        // If page requested is 'all', show all records on one page
        if ($rs_page == 'all') {
            $max_records = 999999;
            $rs_page = 1;
        }
        
        // Get query from SQLBuilder object
        $query = $SQLBuilder->getFullTicketsArray($user_type,
            $search, $field, $sort, $order, $department_ID, $ticket_type); 
            
        // Perform the query
        $rs = &$conn->Execute($query); 
        
        // Build tickets array from query result (use pagination)
        if ($rs != false && $rs->RecordCount() > 0) {
            $ticket_count = 0;
            while (!$rs->EOF) {
                $row = $rs->fields; 
                // Increment ticket count if this ticket is new
                if (!isset($IDs[$row['ID']])) {
                    $ticket_count++;
                } 
                // Only add info to array if ticket is within pagination limits
                if ($ticket_count > (($rs_page - 1) * $max_records) && $ticket_count <= ($rs_page * $max_records)) {
                    // prepare arrays
                    $ticket_fields = array();
                    $user_fields = array();
                    $answer_fields = array();
                    $department_fields = array(); 
                    
                    // Increment 'recent' index count if ticket within the predefined number of days
                    if ($row['timestamp'] >= (time() - OPTION_RECENT_TICKETS_DAYS * 86400)) {
                        $tickets[$row['ID']]['recent'] = true;
                    } else {
                        $tickets[$row['ID']]['recent'] = false;
                    } 

                    foreach ($row as $field => $value) {
                        if (eregi('^user_', $field)) {
                            $user_fields[substr($field, 5)] = $value;
                        } elseif (eregi('^answer_', $field) && $value != null) {
                            $answer_fields[substr($field, 7)] = $value;
                        } elseif (eregi('^department_', $field)) {
                            $department_fields[substr($field, 11)] = $value;
                        } else {
                            $ticket_fields[$field] = $value;
                        } 
                    } 

                    if (count($answer_fields) > 1) {
                        $tickets[$row['ID']]['answers'][] = $answer_fields;
                    } 

                    $tickets[$row['ID']]['ticket'] = $ticket_fields;
                    $tickets[$row['ID']]['user'] = $user_fields;
                    $tickets[$row['ID']]['department'] = $department_fields;
                } 
                $IDs[$row['ID']] = 1;
                $tickets['count'] = $ticket_count;
                $rs->MoveNext();
            } 
            
            return $tickets;
        } else {
            Logger::logEvent(3, "DB Error", null,
                "SQL error trying to get Array of users: " . addslashes($query), __FILE__, __LINE__);
            return false;
        } 
    } 

    /* ---- GETTERS ---- */

    /**
     * Returns the Ticket's user ID.
     * 
     * @return int user_ID
     * @access public 
     */
    public function getUser_ID() {
        return $this->user_ID;
    } 

    /**
     * Returns the Ticket's subject.
     * 
     * @return string Subject
     * @access public 
     */
    public function getSubject() {
        return $this->subject;
    } 

    /**
     * Returns the Ticket's creation timestamp.
     * 
     * @return int Timestamp of creation
     * @access public 
     */
    public function getTimestamp() {
        return $this->timestamp;
    } 

    /**
     * Returns the Ticket's department_ID.
     * 
     * @return int department_ID
     * @access public 
     */
    public function getDepartment_ID() {
        return $this->department_ID;
    } 

    /**
     * Returns the Ticket's status (open or closed).
     * 
     * @return string Status
     * @access public 
     */
    public function getStatus() {
        return $this->status;
    } 

    /**
     * Returns the Ticket's urgency level (1 - 4).
     * 
     * @return int Urgency level
     * @access public 
     */
    public function getUrgency() {
        return $this->urgency;
    } 

    /**
     * Returns the Ticket's category.
     * 
     * @return string Category
     * @access public 
     */
    public function getCategory() {
        return $this->category;
    } 

    /**
     * Returns the Ticket's body.
     * 
     * @return string Body
     * @access public 
     */
    public function getBody() {
        return $this->body;
    } 

    /**
     * Returns the Ticket's answers.
     * 
     * @return array answers
     * @access public 
     */
    public function getAnswers() {
        return $this->answers;
    } 

    /* ---- SETTERS ---- */

    /**
     * Sets the Ticket's ID.
     * 
     * @param int $ID 
     * @access public 
     */
    public function setID($ID) {
        $this->ID = $ID;
    } 

    /**
     * Sets the Ticket's user ID.
     * 
     * @param int $user_ID 
     * @access public 
     */
    public function setUser_ID($user_ID) {
        $this->user_ID = $user_ID;
    } 

    /**
     * Sets the Ticket's subject.
     * 
     * @param string $subject 
     * @access public 
     */
    public function setSubject($subject) {
        $this->subject = $subject;
    } 

    /**
     * Sets the Ticket's creation timestamp.
     * 
     * @param int $timestamp 
     * @access public 
     */
    public function setTimestamp($timestamp) {
        $this->timestamp = $timestamp;
    } 

    /**
     * Sets the Ticket's department_ID.
     * 
     * @param int $department_ID 
     * @access public 
     */
    public function setDepartment_ID($department_ID) {
        $this->department_ID = $department_ID;
    } 

    /**
     * Sets the Ticket's status (open or closed).
     * 
     * @param string $status 
     * @access public 
     */
    public function setStatus($status) {
        $this->status = $status;
    } 

    /**
     * Sets the Ticket's urgency level (1 - 4).
     * 
     * @param int $urgency 
     * @access public 
     */
    public function setUrgency($urgency) {
        $this->urgency = $urgency;
    } 

    /**
     * Sets the Ticket's category.
     * 
     * @param string $category 
     * @access public 
     */
    public function setCategory($category) {
        $this->category = $category;
    } 

    /**
     * Sets the Ticket's body.
     * 
     * @param string $body 
     * @access public 
     */
    public function setBody($body) {
        $this->body = $body;
    } 

    /**
     * Adds an answer to the Ticket.
     * 
     * @param object $answers 
     * @access public 
     */
    public function addAnswer(Answer $answer) {
        $this->answers[] = $answer;
    } 
    
    /**
    * Tostring function: returns a string representation of this object
    * 
    * @access public
    * @return string
    */
    public function __toString() {
        $string = "Ticket object:
            <br />___________________";
        $string .= "<br />ID: " . $this->getID();
        $string .= "<br />Department: " . $this->getDepartment_ID();
        $string .= "<br />Body: " . $this->getBody();
        $string .= "<br />User_ID: " . $this->getUser_ID();
        $string .= "<br />Urgency: " . $this->getUrgency();
        $string .= "<br />Date Created: " . gmdate('d-m-Y', $this->getTimestamp());
        $string .= "<br />Status: " . $this->getStatus();
        $string .= "<br />Answers: <ul>";
        foreach ($this->getAnswers() as $ID => $answer) {
            $string .= "<li>" . $answer->getUser_ID() . ": " . $answer->getBody() . "</li>";
        }
        $string .= "</ul>";
        return $string;
    }
} 

?>
