<?php
/**
 * File containing the User class.
 * 
 * @category Class
 * @package PHPSupportTickets
 * @author Ian Warner, <iwarner@triangle-solutions.com> 
 * @author Nicolas Connault, <nicou@sweetpeadesigns.com.au> 
 * @copyright (C) 2001 Triangle Solutions Ltd
 * @version CVS: $Id: class.User.php,v 1.16 2005/08/19 13:37:27 nicconn Exp $
 * @since File available since Release 1.1.1.1
 * \\||
 */

/**
* user defined includes 
* @include
*/
require_once('class.Note.php');
require_once('abstract.Person.php');
require_once('SQL/class.UserSQLBuilder.php');
require_once('static/static.Logger.php');
/**
* user defined constants 
* @ignore
*/

if (isset($_SESSION['user'])) {
    define('USER_USER_ID', unserialize($_SESSION['user'])->getID());
    define('USER_ADMIN_STATUS', unserialize($_SESSION['user'])->getAdmin_status());
} else {
    define('USER_USER_ID', 0);
    define('USER_ADMIN_STATUS', 'Mod');
}

/**
* User class.
* 
* @access public 
* @package PHPSupportTickets
*/
class User extends Person {
    // --- ATTRIBUTES ---
    
    /**
    * The database table to mirror this object
    * 
    * @access public
    * @static 
    * @var string 
    */
    public static $table = "users";
    
    /**
    * An array of which fields are required from user input
    * 
    * @access public
    * @static
    * @var array
    */
    public static $required_fields = array(
                'username', 'password', 'admin_status',  
                'nickname', 'email');
    
    /**
    * Short description of attribute username
    * 
    * @access private 
    * @var String 
    */
     private $username = null;
    
    /**
    * Short description of attribute password
    * 
    * @access private 
    * @var String 
    */
     private $password = null;        
    
    /**
    * Short description of attribute email
    * 
    * @access private 
    * @var String 
    */
     private $email = null;
    
    /**
    * Short description of attribute lastLogin
    * 
    * @access public 
    * @var String 
    */
     private $last_login = null;
    
    /**
    * Short description of attribute admin_status
    * 
    * @access private 
    * @var boolean 
    */
     private $admin_status = "Mod";
          
     /**
     * Contains all information about this user in an associative array,
     * including all class variables.
     * 
     * @access private
     * @var array info
     */
     private $info = array();
     
     /**
     * Companion object used for building SQL queries. Separates
     * class responsibilities from SQL logic.
     * 
     * @access private
     * @var object $SQLBuilder
     */
     private $SQLBuilder;
     
    // --- METHODS ---
    
    /**
    * Constructor: first call on parent class constructor.
    * 
    * @access public 
    * @param array $fields From a Posted form or a database row 
    * @return void 
    */
    public function __construct($fields) {
        $this->SQLBuilder = new UserSQLBuilder(); 
        if (is_array($fields) && isset($fields['username'])) {
            parent::__construct($fields);
            
            extract($fields);
            $this->setAdmin_status($admin_status);
            $this->setEmail($email);
            $this->setLast_login($last_login);
            $this->setPassword($password); 
            $this->setUsername($username);
            $this->setInfo($fields);
            
            if (isset($ID)) {
                $this->setID($ID);
            }
        } else { 
            Logger::logEvent(3, "User Operation", USER_USER_ID, 
                    "Wrong argument type given to User::__construct", __FILE__, __LINE__);
        }
    } 
    
    /* --- DATABASEMIRROR METHODS --- */
        
    /**
    * This static function must be implemented by all objects that implement
    * the DatabaseMirror interface. It enables us to validate the user input
    * fields BEFORE creating the object or entering it into the DB.
    * 
    * @param array $fields
    * @return boolean True if all fields are valid, array of arrays with error
    *       message and name of field if error is encountered.
    */ 
    public static function validate($fields) {        
        if (is_array($fields)) {
            extract($fields);
            $errors = array();
            
            // Only require non-empty values for required fields for this admin type
            $admin = USER_ADMIN_STATUS;
            
            // First retrieve parent class validate() errors, if any
            if (parent::validate($fields) !== true) {
                $errors = parent::validate($fields);
            }
            
            // Verify that required fields are not empty
            
            if (in_array("email", User::$required_fields) &&
                    ($email == "" || is_null($email))) {
                $errors[] = array("field" => "email", 
                        "message" => VALIDATE_EMPTY_EMAIL);
            } elseif (Validate::checkEmail($email) !== true) {
                $errors[] = Validate::checkEmail($email, "email");
            } 
            
            if (in_array("username", User::$required_fields) &&
                    ($username == "" || is_null($username))) {
                $errors[] = array("field" => "username", 
                        "message" => VALIDATE_EMPTY_USERNAME);
            } elseif (Validate::checkAlphanumeric($username) !== true) {
                $errors[] = Validate::checkAlphanumeric($username, "username");
            }
            
            if (in_array("password", User::$required_fields) &&
                    ($password == "" || is_null($password))) {
                $errors[] = array("field" => "password", 
                        "message" => VALIDATE_EMPTY_PASSWORD);
            } elseif (Validate::checkPassword($password) !== true) {
                $errors[] = Validate::checkPassword($password, "password");
            }
            
            if (in_array("admin_status", User::$required_fields) 
                   && (!isset($admin_status)|| $admin_status == "")) {
                $errors[] = array("field" => "admin_status", 
                        "message" => VALIDATE_EMPTY_ADMIN_STATUS);
            } elseif ($admin_status != "Admin" 
                    && $admin_status != "Mod" 
                    && $admin_status != "Client") {
                $errors[] = array("field" => "admin_status", 
                        "message" => VALIDATE_INVALID_ADMIN_STATUS);            
            }             
            
            if (count($errors) == 0) {
                Logger::logEvent(3, "User Operation", USER_USER_ID, 
                        "User object created, passed validation", __FILE__, __LINE__);
                return true;
            } else {           
                Logger::logEvent(3, "User Operation", USER_USER_ID, 
                        "User object not created, failed validation", __FILE__, __LINE__);
                return $errors;
            }
        }
    }
    
    /**
    * Retrieves this User's ID number based on a unique variable.
    * Also sets this object's ID number to the retrieved int
    * 
    * @return int User's ID number
    */
    public function getID() {
        $conn = &ADONewConnection(DSN);
        
        // We might be searching for a Client with no username, so look
        // for another unique combination of fields.
        if (is_null($this->getUsername()) || $this->getUsername() == "") {
            $query = $this->SQLBuilder->search("users", array("ID"), 
                    array("email" => $this->getEmail(), "nickname" => $this->getNickname()));
        } else {
            $query = $this->SQLBuilder->search("users", array("ID"), 
                array("username" => $this->getUsername()));
        }
        
        $rs = &$conn->Execute($query);
        
        if ($rs != false && $rs->RecordCount() > 0) {
            $this->ID = $rs->fields['ID'];
            return $this->ID;            
        } else {
            Logger::logEvent(3, "DB Error", USER_USER_ID, 
                    "Could not retrieve User ID: " . $this->getUsername(), __FILE__, __LINE__);
            return false;
        }
    }
    
    /**
    * Short description of method add
    * 
    * @access public 
    * @return boolean 
    */
    public function addToDB() {
        // Initialise Database object
        $conn = &ADONewConnection(DSN);
        
        // First check that User doesn't already exist.
        if (is_null($this->getUsername()) || $this->getUsername() == "") {
            $query = $this->SQLBuilder->search("users", array("ID"), 
                    array("email" => $this->getEmail(), "nickname" => $this->getNickname()));
        } else {
            $query = $this->SQLBuilder->search("users", array("ID"), 
                array("username" => $this->getUsername()));
        }
        $rs = &$conn->Execute($query);
        
        if ($rs->RecordCount() == 0) {
            // Retrieve array of table's column names (always UPPERCASE)
            $columns = $conn->MetaColumnNames("users");
            $query = $this->SQLBuilder->insert($this, $columns);
            $rs = &$conn->Execute($query);
            
            // Verify previous query
            if ($rs != false) {
                Logger::logEvent(3, "User Operation", USER_USER_ID, 
                        "User object added to DB", __FILE__, __LINE__);
                return true;
            } else {                
                Logger::logEvent(3, "DB Error", USER_USER_ID, 
                        "Could not Insert User object.", __FILE__, __LINE__);
                return false;
            }
        } else {
            Logger::logEvent(3, "User Operation", USER_USER_ID, 
                        "User object not added to DB, username already exists", __FILE__, __LINE__);
            return null; 
        }
    }
    
    /**
    * Removes a User from the Database
    * 
    * @access public 
    * @param int $ID
    * @return boolean true if successful
    */
    public function removeFromDB($ID) {
        $conn = &ADONewConnection(DSN);
        $query = $this->SQLBuilder->delete($ID);
        $rs = &$conn->Execute($query);
        
        if ($conn->Affected_Rows() > 0) {
            Logger::logEvent(3, "User Operation", USER_USER_ID, 
                    "User object deleted from DB.", __FILE__, __LINE__);
            
            // Now clean up the associated user_contacts table
            $query = $this->SQLBuilder->deleteContact($this->getID());
            $rs = &$conn->Execute($query);            
            return true;
        } else {
            Logger::logEvent(3, "DB Error", USER_USER_ID, 
                    "User object could not be deleted", __FILE__, __LINE__);
            return false;
        }
    }
    
    /**
    * Updates a User in the Database
    * 
    * @access public 
    * @param int
    * @return boolean true if successful
    */
    public function updateDB() {
        $conn = &ADONewConnection(DSN);
        $this->getID();
        $Contact = null;
        $columns = $conn->MetaColumnNames(User::$table);
        $query = $this->SQLBuilder->update($this, $columns);
        
        $rs = &$conn->Execute($query);
        
        if ($rs != false && $conn->Affected_Rows() > 0) {
            Logger::logEvent(3, "User Operation", USER_USER_ID, 
                    "User object updated in DB", __FILE__, __LINE__);
        } elseif($conn->Affected_Rows() == 0) {                
            Logger::logEvent(3, "User Operation", USER_USER_ID, 
                    "User object note updated, no changes to perform", __FILE__, __LINE__);
        } else {            
            Logger::logEvent(3, "DB Error", USER_USER_ID, 
                    "User object not updated, DB error", __FILE__, __LINE__);
            return false;
        }
       
        // Update department_users table if the user has a department_ID
        if ($this->getInfo('department_ID') > 0) {
            $query = $this->SQLBuilder->updateDepartment_users($this->getID(), 
                    $this->getInfo('department_ID'));
            $rs = &$conn->Execute($query);
            
            if ($rs != false) {
                Logger::logEvent(3, "Department Operation", USER_USER_ID, 
                        "User reference updated in department_users table", __FILE__, __LINE__);
            } else {
                Logger::logEvent(3, "DB Error", USER_USER_ID, 
                        "Failed to update department_users table", __FILE__, __LINE__);
                return false;
            }
        }
        return true;
    }
    
    /**
     * Implementation of the DatabaseMirror interface function getFromDB().
     * Returns a User object from the DB based on input in $fields array
     * 
     * @static
     * @access public
     * @return Object User if successful, false otherwise
     */    
    public static function getFromDB($fields) {              
        if (is_array($fields)) {
            $myUser = null;
            
            $conn = &ADONewConnection(DSN);
            $conn->SetFetchMode(ADODB_FETCH_ASSOC);
            $SQLBuilder = new UserSQLBuilder();
            $query = $SQLBuilder->get($fields);

            if (!$query) {
                Logger::logEvent(3, "User Operation", USER_USER_ID, 
                    "Wrong arguments passed to User::getFromDB()", __FILE__, __LINE__);
            }
            
            $rs = &$conn->Execute($query);
            
            if ($rs != false && $rs->RecordCount() > 0) {
                Logger::logEvent(3, "User Operation", USER_USER_ID, 
                        "User object retrieved from DB", __FILE__, __LINE__);
                $myUser = new User($rs->fields);
            } else {
                Logger::logEvent(3, "DB Error", USER_USER_ID, 
                        "User object could not be retrieved from DB", __FILE__, __LINE__);
                return false;
            }
        } else {
            Logger::logEvent(3, "User Operation", USER_USER_ID, 
                    "Wrong argument type passed to User::getFromDB()", __FILE__, __LINE__);
            return false;
        }
        return $myUser;        
    }
    
    /**
    * Returns an array of user objects, fully instantiated with
    * department info and address info, using one large SQL query.
    * To do this, we get all the info we can from the query, and use
    * only object constructors (which do not open DB connections)
    * 
    * @access public
    * @static
    * 
    * @param string $user_type Admin or Mod
    * @param string $search The search term
    * @param string $field The field being searched
    * @param string $sort The field by which to sort the results
    * @param string $order The order in which to sort the results
    * 
    * @return array of Users
    */
    public static function getArrayFromDB($user_type = "Mod", $search = "", $field = ""
            , $sort = "", $order = "", $max_records, $rs_page) {
        $conn = &ADONewConnection(DSN);
        $SQLBuilder = new UserSQLBuilder();
        $users = array();
        $IDs = array();
        if (!isset($rs_page)) {
            $rs_page = 1;
        }
        $users['count'] = 0;
        
        // Get query for retrieving all users from DB
        $query = $SQLBuilder->getFullUsersArray($user_type, $search, $field, $sort, $order);
        
        // Perform the query
        $rs = &$conn->Execute($query);
        
        // If page requested is 'all', show all records on one page
        if ($rs_page == 'all') {
            $max_records = 999999;
            $rs_page = 1;
        }
        
        // Build Users array from query result
        if ($rs != false && $rs->RecordCount() > 0) {
           $user_count = 0;
           while (!$rs->EOF) {
               $row = $rs->fields;
               // die(print $rs->RecordCount());
               
               // Increment user count if this ticket is new
               if (!isset($IDs[$row['ID']])) {
                   $user_count++;
               }
               
               // Only add info to array if user is within pagination limits
               if ($user_count > (($rs_page - 1) * $max_records) 
                        && $user_count <= ($rs_page * $max_records)) {
                   
                   $user_fields = array();
                   $ticket_fields = array();
                   $department_fields = array();
                   
                   foreach ($row as $field => $value) {
                       if (eregi('^ticket_', $field) && $value != null) {
                           $ticket_fields[substr($field, 7)] = $value;
                       } elseif (eregi('^department_', $field) && $value != null) {
                           $department_fields[substr($field, 11)] = $value;
                       } else {
                           $user_fields[$field] = $value;
                       }
                   }
                   
                   if (count($ticket_fields) > 1) {
                       $users[$row['ID']]['tickets'][] = $ticket_fields;
                   }
                   $users[$row['ID']]['user'] = $user_fields;
                   $users[$row['ID']]['department'] = $department_fields;
               }
               
               $IDs[$row['ID']] = 1;
               $rs->MoveNext();
               
               $users['count'] = $user_count;
           }
           
           return $users;
        } else {
            Logger::logEvent(3, "DB Error", USER_USER_ID, 
                    "SQL error trying to get Array of users: " . addslashes($query), __FILE__, __LINE__);
            return false;
        }
    }
    
    /**
    * Returns a string of this user's descriptive information.
    * 
    * @return string
    */
    public function getFullDescription() {
        return $this->getFname() . " " 
                . $this->getLname();
    }
    
    /* ---- GETTERS ---- */
    
    /**
    * Returns the User's info chosen field.
    * 
    * @return array
    * @access public
    * @param string field
    */
    public function getInfo($field) {
        if (array_key_exists($field, $this->info)) {
            return $this->info[$field];
        } else {
            return null;
        }
    }
    
    /**
    * Returns the User's username.
    * 
    * @return string
    * @access public
    */
    public function getUsername() {
        return $this->username;
    }
        
    /**
    * Returns the User's encrypted password.
    * 
    * @return string
    * @access public
    */
    public function getPassword() {
        return $this->password;
    }
        
    /**
    * Returns the User's email address.
    * 
    * @return string
    * @access public
    */
    public function getEmail() {
        return $this->email;
    }
    
    /**
    * Returns the User's last login UNIX Timestamp.
    * Needs formatting by a DateHandler class
    * 
    * @return string
    * @access public
    */
    public function getLast_login() {
        return $this->last_login;
    }
    
    /**
    * Returns the User's admin status.
    * This has 3 possible settings, determining what level of 
    * control the User has. They are 'Admin', 
    * 'Mod'and 'Client'. Admins and Mods can only belong to
    * the admin company. Clients can only belong to other companies.
    * 
    * @return string
    * @access public
    */
    public function getAdmin_status() {
        return $this->admin_status;
    }

    /* ---- SETTERS ---- */
    
    /**
    * Sets the User's username.
    * 
    * @return boolean True if successful, false otherwise.
    * @access public
    * @param string
    */
    public function setUsername($string) {
        // @todo Do some validation first
        $this->username = $string;
    }
    
    /**
    * Sets the User's info array.
    * 
    * @return boolean True if successful, false otherwise.
    * @access public
    * @param array
    */
    public function setInfo($array) {
        // @todo Do some validation first
        $this->info = $array;
    }
    
    /**
    * Sets the User's encrypted password.
    * 
    * @return boolean True if successful, false otherwise.
    * @access public
    * @param string
    */
    public function setPassword($string) {
        // @todo Do some validation first
        $this->password = $string;
    }
        
    /**
    * Sets the User's email address.
    * 
    * @return boolean True if successful, false otherwise.
    * @access public
    * @param string
    */
    public function setEmail($string) {
        // @todo Do some validation first
        $this->email = $string;
    }
    
    /**
    * Sets the User's last login UNIX Timestamp.
    * Needs formatting by a DateHandler class
    * 
    * @return boolean True if successful, false otherwise.
    * @access public
    * @param int
    */
    public function setLast_login($timestamp) {
        // @todo Do some validation first
        $this->last_login = $timestamp;
    }
    
    /**
    * Sets the User's admin status.
    * This has 3 possible settings, determining what level of 
    * control the User has. They are 'Admin', 
    * 'Mod'and 'Client'. Admins and Mods can only belong to
    * the admin company. Clients can only belong to other companies.
    * 
    * @return boolean True if successful, false otherwise.
    * @access public
    * #param string
    */
    public function setAdmin_status($string) {
        // @todo Do some validation first
        $this->admin_status = $string;
    }
    
    /**
    * Tostring function: returns a string representation of this object
    * 
    * @access public
    * @return string
    */
    public function __toString() {
        $string = "User object:
            <br />___________________";
        $string .= "<br />ID: " . $this->getID();
        $string .= "<br />Name: " . $this->getNickname();
        $string .= "<br />username: " . $this->getUsername();
        $string .= "<br />password: " . $this->getPassword();
        $string .= "<br />email: " . $this->getEmail();
        $string .= "<br />Last Login: " . gmdate('d-m-Y', $this->getLast_login());
        $string .= "<br />Admin Status: " . $this->getAdmin_status();
        return $string;
    }

} 

?>
